
%%%%%% 7) Radial profile of magnitudes_left
clear all

function [y] = line_func(x, p)
    % p = vector of parameters to be adjusted
    % x = vector of observations
    y = p(1) + p(2) * x(:,1);
end


cd output
galaxy_magnitudes
cd ..

%%%%%%%%%%%%%%%%
%%%% Left side
pin=[0, 0.75];

weights = ones(length(y_magnitudes_left),1);
[f,p,kvg,iter,corp,covp,covr,stdresid,Z,r2]=leasqr(x_magnitudes_left,y_magnitudes_left,pin,"line_func",stol=.00001,niter=20,wt=weights);

p_left = p;

%Uncertainty = Standard deviation
u_left = sqrt(diag(covp));


%%%%%%%%%%%%%%%%
%%%% Right side
pin=[0, 0.25];

weights = ones(length(y_magnitudes_right),1);
[f,p,kvg,iter,corp,covp,covr,stdresid,Z,r2]=leasqr(x_magnitudes_right,y_magnitudes_right,pin,"line_func",stol=.00001,niter=20,wt=weights);

%Uncertainty = Standard deviation
u = sqrt(diag(covp));

% Left:   y = k1 + s1 * mag_break
% Right:  y = k2 + s2 * mag_break
% k1 + s1 * mag_break = k2 + s2 * mag_break
% s1 * mag_break - s2 * mag_break = k2 - k1
% mag_break * (s1 - s2) = k2 - k1
mag_break_obs = (p_left(1) - p(1)) / (p(2) - p_left(2));
y_break_obs = line_func(mag_break_obs,p);
y_break = line_func(mag_break,p); % Theoretical

%%%%%%%%%%%%%%%%
%%%% Full plot
x_magnitudes = [x_magnitudes_left; x_magnitudes_right];
y_magnitudes = [y_magnitudes_left; y_magnitudes_right];

clf
xx = [8.30:1:22];
xx_limited = [mag_break_obs:1:22];
axis([8.30, 22, 0, 3], "square");
hold("on")
plot(xx_limited,line_func(xx_limited(:),p),'-3',xx,line_func(xx(:),p_left),'-3', x_magnitudes(:,1),y_magnitudes,'@1', mag_break_obs, y_break_obs, '@3', x_magnitudes_rest, y_magnitudes_rest, '@0')
title('Magnitude function');
xlabel('Magnitude v');
ylabel('log N(<V)');
grid on
print("output/magnitude_function.png","-dpng", "-S800,600")


disp("")
disp("Funció de lluminositat:")
disp("\t- Creuament teòric: ")
disp(strcat("\t\tMagnitud = ", num2str(mag_break)))
disp(strcat("\t\tlog(N(<m)) = ", num2str(y_break)))
disp("\t- Creuament observacional: ")
disp(strcat("\t\tMagnitud = ", num2str(mag_break_obs)))
disp(strcat("\t\tlog(N(<m)) = ", num2str(y_break_obs)))
disp(strcat("\t- Fórmula:"))
disp(strcat("\t\tlog(N(<m)) = k1 + s1 * m\t\tsi m <= ", num2str(mag_break_obs)))
disp(strcat("\t\tlog(N(<m)) = k2 + s2 * m\t\tsi m > ", num2str(mag_break_obs)))
disp("\t- Paràmetres:")
disp(strcat("\t\tk1 = ", num2str(p_left(1))))
disp(strcat("\t\ts1 = ", num2str(p_left(2))))
disp(strcat("\t\tk2 = ", num2str(p(1))))
disp(strcat("\t\ts2 = ", num2str(p(2))))
disp("\t- Desviació estàndard:")
disp(strcat("\t\tk1 = ", num2str(u_left(1))))
disp(strcat("\t\ts1 = ", num2str(u_left(2))))
disp(strcat("\t\tk2 = ", num2str(u(1))))
disp(strcat("\t\ts2 = ", num2str(u(2))))
disp("")


%%%%%%%%%%%%%%%%
%%%% Luminosity
clf
xx = [8.30:1:22];
plot(x_luminosity(:,1),y_luminosity,'@1')
title('Luminosity function');
xlabel('Luminosity B');
ylabel('N(<Lb)');
grid on
print("output/luminosity_function.png","-dpng", "-S800,600")


