
%%%%%% 7) Radial profile of density
%%%%%%    - Non-Linear fit
clear all

% Function to fit
function [y] = line_func(x, p)
    % p = vector of parameters to be adjusted
    % x = vector of observations
    y = p(1) * (1 + (x(:,1) / p(3)).**2).**(-p(2));
end

% Load data
cd output
rings
cd ..

% Initial parameter's value
pin=[1, 1, 0.1];

% Paramter logical bounds
options.bounds = [-Inf, Inf; -Inf, Inf; 0.1, 0.5];

% It is possible to implement the exact derivative of the function to fit or use a generic one (less efficient)
%dFdp = leasqrdfdp; % exact derivative
dFdp = "dfdp";     % estimated derivative

% Fractional increment of p for numerical partial derivatives
dp = .001*ones(size(pin));

% Tolerance on fractional improvement
stol=.00001;

% Maximum number of iterations
niter=20;

% Statistical weights
weights = ones(length(y_density),1);

[f,p,kvg,iter,corp,covp,covr,stdresid,Z,r2]=leasqr(x_density,y_density,pin,"line_func",stol, niter,weights, dp, dFdp, options);


%Uncertainty = Standard deviation
u = sqrt(diag(covp));

clf
xx = [0:0.1:3];
plot(xx,line_func(xx(:),p),'-3', x_density(:,1),y_density,'@1')
title('Radial profile of density');
xlabel('Radius');
ylabel('Density');
grid on
print("output/radial_profile_density.png","-dpng", "-S800,600")

disp("")
disp("Perfil radial de densitat:")
disp("\t- Fórmula: s(r) = s0 * (1 + (r / r0)^2)^(-a)")
disp("\t- Paràmetres:")
disp(strcat("\t\ts0 = ", num2str(p(1))))
disp(strcat("\t\ta = ", num2str(p(2))))
disp(strcat("\t\tr0 = ", num2str(p(3))))
disp("\t- Desviació estàndard:")
disp(strcat("\t\ts0 = ", num2str(u(1))))
disp(strcat("\t\ta = ", num2str(u(2))))
disp(strcat("\t\tr0 = ", num2str(u(3))))
disp("")


%%%%%% 7) Radial profile of density
%%%%%%    - Linear fit
clear all

function [y] = line_func(x, p)
    % p = vector of parameters to be adjusted
    % x = vector of observations
    y = p(1) - p(2) * x(:,1);
end

cd output
rings
cd ..

pin=[45, 1];

weights = ones(length(y_density_linear),1);
[f,p,kvg,iter,corp,covp,covr,stdresid,Z,r2]=leasqr(x_density_linear,y_density_linear,pin,"line_func",stol=.00001,niter=20,wt=weights);

%Uncertainty = Standard deviation
u = sqrt(diag(covp));

clf
xx = [0:0.1:2];
plot(xx,line_func(xx(:),p),'-3', x_density_linear(:,1),y_density_linear,'@1')
title('Radial profile of density');
xlabel('log(1 + (Radius / 0.5)^2)');
ylabel('log(Density)');
grid on
print("output/radial_profile_density_linear.png","-dpng", "-S800,600")

disp("")
disp("Perfil radial de densitat (validació lineal):")
disp("\t- Fórmula: log(s(r)) = log(s0) - a*log(1 + (r / 0.5)^2)")
disp("\t- Paràmetres:")
disp(strcat("\t\tlog(s0) = ", num2str(p(1))))
disp(strcat("\t\ta = ", num2str(p(2))))
disp("\t- Desviació estàndard:")
disp(strcat("\t\tlog(s0) = ", num2str(u(1))))
disp(strcat("\t\ta = ", num2str(u(2))))
disp("")



%%%%%% 8) Radial profile of velocity dispersion
clear all

function [y] = line_func(x, p)
    % p = vector of parameters to be adjusted
    % x = vector of observations
    y = p(1) + p(2) * x(:,1);
end


cd output
rings
cd ..

pin=[1, 1];

weights = ones(length(y_vel),1);
[f,p,kvg,iter,corp,covp,covr,stdresid,Z,r2]=leasqr(x_vel,y_vel,pin,"line_func",stol=.00001,niter=20,wt=weights);

%Uncertainty = Standard deviation
u = sqrt(diag(covp));

clf
xx = [0:0.1:3];
plot(xx,line_func(xx(:),p),'-3', x_vel(:,1),y_vel,'@1')
title('Radial profile of velocity dispersion');
xlabel('Radius');
ylabel('Velocity dispersion');
grid on
print("output/radial_profile_velocity_dispersion.png","-dpng", "-S800,600")

disp("Perfil radial de velocitat de dispersió:")
disp("\t- Fórmula: w(r) = a + b*r")
disp("\t- Paràmetres:")
disp(strcat("\t\ta = ", num2str(p(1))))
disp(strcat("\t\tb = ", num2str(p(2))))
disp("\t- Desviació estàndard:")
disp(strcat("\t\ta = ", num2str(u(1))))
disp(strcat("\t\tb = ", num2str(u(2))))
disp("")
