#!/usr/bin/ruby
require 'open3'
#require 'rubygems'
#require 'ruby-debug' 
#Debugger.start(:post_mortem => true)

class Cumul
    include Math

    ## Load file
    def initialize(fitxer)
        infile = File.new(fitxer, "r")
        @linies = []
        
        infile.each { |i|
            @linies.push i.split(" ")
        }
        infile.close
        calculate_attributes
    end

    #### TASK 1
    ## Filter loaded galaxies, discarding those which are not valid for the analysis
    def clean
        print_vel_attributes
        filter_zero_or_no_member
        puts ":: Filtrat de velocitats zero i no membres"
        print_vel_attributes
        save("output/VIRGODOS_zero_filtered.txt")

        begin
            n = @num_galaxies
            filter_3sigma
            nou_n = @num_galaxies
            puts ":: Filtrat de velocitats > 3 x sigma"
            print_vel_attributes
        end while nou_n - n != 0
        save("output/VIRGODOS_zero_and_3sigma_filtered.txt")
    end
    
    ## Save galaxies
    def save(fitxer)
        outfile = File.new(fitxer, "w")
        
        ## Print field names
        fields = @f.sort{|a,b| a[1] <=> b[1]}
        fields.each { |field|
            outfile.write field[0] + "(" + field[1].to_s + ") "
        }
        outfile.write "\n"
        
        @linies.each { |l|
            outfile.write l.join(" ") + "\n"
        }
        outfile.close
    end
    
    #### TASK 2
    def calculate_vel_caiguda
        @vel_caiguda = 0.0
        @vel_caiguda_err = 0.0
        @ra_mig = 0.0
        @dec_mig = 0.0
        @linies.each { |l|
            ## Right ascension
            # Convert to hours
            ra = l[f("RA_h")].to_f + (l[f("RA_m")].to_f / 60) + (l[f("RA_s")].to_f / (60*60))
            # to radians
            ra = ra * (360.0/24) * (PI / 180)
            @ra_mig += ra
            
            ## Declination
            # Convert to degrees
            dec = l[f("DEC_g")].to_f + (l[f("DEC_m")].to_f / 60) + (l[f("DEC_s")].to_f / (60*60))
            # to radians
            dec = dec * (PI / 180)
            @dec_mig += dec

            ### Equatorial to galactic coordinates
            ## Latitude: b
            # sin B = sin (dec) cos (62º.6) -cos (dec) sin (62º.6) sin (A.R. - 282º.25)
            b = asin(sin(dec) * cos(62.6 * (PI/180)) - cos(dec) * sin(62.6 * (PI/180)) * sin(ra - 282.25 * (PI/180)))
            ## Longitude: l
            # cos B sin (L - 33º.012) = sin (dec) sin (62º.6) + cos (dec) cos (62º.6) sin (A.R. - 282º.5)
            p = sin(dec) * sin(62.6 * (PI/180)) + cos(dec) * cos(62.6 * (PI/180)) * sin(ra - 282.25 * (PI/180))
            # cos B cos (L - 33º.012) = cos (dec) cos (A.R. - 282º.85)
            q = cos(dec) * cos(ra - 282.25 * (PI/180))
            
            # Corrections
            lFinal = atan2(p, q) + (33.012 * (PI/180))
            if (lFinal < 0)
                lFinal += 2*PI
            end
            
            l.push b
            @f["Galactic_latitude"] = l.size - 1
            l.push lFinal
            @f["Galactic_longitude"] = l.size - 1

            ## Corrected velocity
            vel = (360 * cos(lFinal - 265 * (PI/180)) * cos(b - 50 * (PI/180))) + l[f("Heliocentric_velocity")].to_f
            vel_err = sqrt(27**2 + l[f("Heliocentric_velocity_error")].to_f**2)
            l.push ra
            @f["RA"] = l.size - 1
            l.push dec
            @f["DEC"] = l.size - 1
            l.push vel
            @f["Velocity"] = l.size - 1
            
            @vel_caiguda += vel
            @vel_caiguda_err += vel_err
        }
        @vel_caiguda /= @num_galaxies
        @vel_caiguda_err /= @num_galaxies
        @distancia = @vel_caiguda / 71 # Hubble law
        @distancia_err = @vel_caiguda_err / 71
        
        # Virgo cluster center
        @ra_mig /= @num_galaxies
        @dec_mig /= @num_galaxies
        
        puts "Vel caiguda:\t\t" + @vel_caiguda.to_s + " km/s"
        puts "Vel caiguda (error):\t" + @vel_caiguda_err.to_s + " km/s"
        puts "Distància:\t\t" + @distancia.to_s + " Mpc"
        puts "Distància (error):\t" + @distancia_err.to_s + " Mpc"
    end
    
    #### TASK 3
    def calculate_xy
        # Reference point: virgo center
        puts "Centre (X = 0, Y = 0):\t(R.A. = " + (@ra_mig*(180/PI)*(24.0/360)).to_s + "H, Dec = " + (@dec_mig*(180/PI)).to_s + "º)"
        @baricenter_X = 0.0
        @baricenter_Y = 0.0
        @linies.each { |l|
            ra = l[f("RA")]
            dec = l[f("DEC")]
            #d = @distance * sqrt((ra - @ra_mig)**2 + (dec - @dec_mig)**2)
            x = @distancia * (ra - @ra_mig)
            y = @distancia * (dec - @dec_mig)
            l.push x
            @f["X"] = l.size - 1
            l.push y
            @f["Y"] = l.size - 1
            @baricenter_X += x
            @baricenter_Y += y
        }
        @baricenter_X /= @num_galaxies
        @baricenter_Y /= @num_galaxies
        puts "Baricentre:\t\t(X = " + @baricenter_X.to_s + " Mpc, Y = " + @baricenter_Y.to_s + " Mpc)"
    end
    
    #### TASK 4
    def calculate_ellipticity_and_orientation
        a11 = 0
        a22 = 0
        a12_21 = 0
        @linies.each { |l|
            x = l[f("X")]
            y = l[f("Y")]
            a11 += (x - @baricenter_X)**2
            a22 += (y - @baricenter_Y)**2
            a12_21 += (x - @baricenter_X) * (y - @baricenter_Y)
        }
        a11 /= @num_galaxies
        a22 /= @num_galaxies
        a12_21 /= @num_galaxies
        # Valors propis:
        vp1 = (a11 + a22 + sqrt((a11+a22)**2 - 4*(a11*a22 - a12_21**2))) / 2
        vp2 = (a11 + a22 - sqrt((a11+a22)**2 - 4*(a11*a22 - a12_21**2))) / 2
        # Source: http://en.wikipedia.org/wiki/Image_moment#Examples_2
        @ellipticity = sqrt((vp1 - vp2) / vp1)
        puts "El.lipticitat:\t\t" + @ellipticity.to_s
        
        # Vectors propis:
        outfile_eigendata = File.new("output/eigenvectors_data.m", "w")
        outfile_eigendata.write "A = [" + a11.to_s + ", " + a12_21.to_s + "; " + a12_21.to_s + ", " + a22.to_s + "];"
        outfile_eigendata.close
        
        stdin, stdout, stderr = Open3.popen3('/usr/bin/octave --silent eigenvectors.m')
        s = stdout.readlines
        axis_1_rotation_angle = s[0].chop
        axis_2_rotation_angle = s[1].chop

        puts "Orientació el.lipsoide: " + axis_1_rotation_angle.to_s + " graus i " + axis_2_rotation_angle.to_s + " graus respecte l'eix X original"
    end
    
    #### TASK 5
    def calculate_lambda
        lambda_plus_ES0 = 0.0
        lambda_minus_ES0 = 0.0
        num_ES0 = 0
        lambda_plus_IS = 0.0
        lambda_minus_IS = 0.0
        num_IS = 0
        lambda_plus_bigmag = 0.0
        lambda_minus_bigmag = 0.0
        num_bigmag = 0
        lambda_plus_smallmag = 0.0
        lambda_minus_smallmag = 0.0
        num_smallmag = 0
        @linies.each { |l|
            x = l[f("X")]
            y = l[f("Y")]
            r = sqrt(x**2 + y**2)
            if (["E","dE","S0","dS0"].include?(l[f("Type")])) then
                lambda_plus_ES0 += r
                lambda_minus_ES0 += r**-1
                num_ES0 += 1
            else 
                lambda_plus_IS += r
                lambda_minus_IS += r**-1
                num_IS += 1
            end
            
            if (l[f("Apparent_magnitude_B")].to_f >= @mag_mitja) then
                lambda_plus_bigmag += r
                lambda_minus_bigmag += r**-1
                num_bigmag += 1
            else
                lambda_plus_smallmag += r
                lambda_minus_smallmag += r**-1
                num_smallmag += 1
            end
        }
        lambda_plus_ES0 /= num_ES0
        lambda_minus_ES0 /= num_ES0
        lambda_plus_IS /= num_IS
        lambda_minus_IS /= num_IS
        puts "Tipus E+S0:"
        puts "\tLambda+1:\t" + lambda_plus_ES0.to_s
        puts "\tLambda-1:\t" + lambda_minus_ES0.to_s
        puts "\tNum:\t" + num_ES0.to_s
        puts "Tipus I+S+?"
        puts "\tLambda+1:\t" + lambda_plus_IS.to_s
        puts "\tLambda-1:\t" + lambda_minus_IS.to_s
        puts "\tNum:\t" + num_IS.to_s
        
        lambda_plus_bigmag /= num_bigmag
        lambda_minus_bigmag /= num_bigmag
        lambda_plus_smallmag /= num_smallmag
        lambda_minus_smallmag /= num_smallmag
        puts "Mag >= " + @mag_mitja.to_s + ":\t"
        puts "\tLambda+1:\t" + lambda_plus_bigmag.to_s
        puts "\tLambda-1:\t" + lambda_minus_bigmag.to_s
        puts "\tNum:\t" + num_bigmag.to_s
        puts "Mag < " + @mag_mitja.to_s + ":\t"
        puts "\tLambda+1:\t" + lambda_plus_smallmag.to_s
        puts "\tLambda-1:\t" + lambda_minus_smallmag.to_s
        puts "\tNum:\t" + num_smallmag.to_s
    end
    
    #### TASK 6 + 7
    ## Radial profiles
    def calculate_rings
        @baricenter_X
        @baricenter_Y
        @linies.each { |l|
            x = l[f("X")]
            y = l[f("Y")]
            # distance to baricenter
            d = sqrt(x**2+y**2)
            l.push d # 18
            @f["Distance_to_baricenter"] = l.size - 1
        }
        
        # Sort by distance to baricenter (farthest first)
        linies_ord = @linies.sort {|a,b| b[f("Distance_to_baricenter")]<=>a[f("Distance_to_baricenter")]}
        n_rings = 10 # Number of rings
        inc = linies_ord.size / n_rings
        last_pos_int = nil
        outfile_density = File.new("output/rings_density.dat", "w")
        outfile_vel = File.new("output/rings_velocity_dispersion.dat", "w")
        octave_x_density = ""
        octave_y_density = ""
        octave_x_density_linear = ""
        octave_y_density_linear = ""
        octave_x_vel = ""
        octave_y_vel = ""
        for i in 0..n_rings-1
            pos_ext = i*inc # Posició de la galàxia més externa de l'anell
            pos_int = i*inc + (inc-1)
            if pos_int >= linies_ord.size then
                pos_int = linies_ord.size - 1
            end
            

            if last_pos_int.nil? then
                r_ext = linies_ord[pos_ext][f("Distance_to_baricenter")]
            else
                r_ext = linies_ord[last_pos_int][f("Distance_to_baricenter")]
            end
            
            if (pos_int + inc) >= linies_ord.size - 1 then
                r_int = 0
                pos_int = linies_ord.size - 1
            else 
                r_int = linies_ord[pos_int][f("Distance_to_baricenter")]
            end
            
            ## 6) Radial profile of density
            s = PI * r_ext**2 - PI * r_int**2
            n = pos_int - pos_ext + 1
            r_central = r_ext-(r_ext-r_int)/2
            densitat = n/s
            puts "Anell " + (n_rings-i).to_s
            puts "\tRadi exterior:\t\t\t" + r_ext.to_s + " Mpc"
            puts "\tRadi central:\t\t\t" + (r_central).to_s + " Mpc"
            puts "\tÀrea:\t\t\t\t" + s.to_s  + " Mpc^2"
            puts "\tNum. galàxies:\t\t\t" + n.to_s
            puts "\tDensitat (sigma):\t\t" + (densitat).to_s + " Galàxies/Mpc^2"
            puts "\tEpsilon sigma:\t\t\t" + (sqrt(n)/s).to_s
            
            ## Save density in #/Mpc^2 and central radius in Mpc
            outfile_density.write  r_central.to_s + "\t" + densitat.to_s + "\n"
            octave_y_density += densitat.to_s + ";"
            octave_x_density += r_central.to_s + ";"
            octave_y_density_linear += log10(densitat).to_s + ";"
            octave_x_density_linear += log10(1 + (r_central / 0.5)**2).to_s + ";"
            ## Plot with:
            ##   gnuplot> plot 'rings.dat' using 2:1 with line
            
            ## 7) Radial profile of velocity dispersion
            w = 0
            for j in pos_ext..pos_int
                v = linies_ord[j][f("Velocity")]
                w += (v - @vel_caiguda)**2
            end
            w /= n
            w = sqrt(w)
            ew = w / (sqrt(2*n))
            puts "\tVelocitat de dispersió:\t\t" + w.to_s
            puts "\tEpsilon vel. de dispersió:\t" + ew.to_s

            outfile_vel.write r_central.to_s + " " + w.to_s + "\n"
            octave_y_vel += w.to_s + "; "
            octave_x_vel += r_central.to_s + "; "
            
            ## Save associated ring to every galaxy
            for j in pos_ext..pos_int
                l = @linies.find {|l| l[f("VCC")] == linies_ord[j][f("VCC")] }
                l.push n_rings-i
                @f["Ring"] = l.size - 1
            end
            
	        last_pos_int = pos_int
        end
        outfile_octave = File.new("output/rings.m", "w")
        outfile_octave.write "x_density = [" + octave_x_density + "];\n"
        outfile_octave.write "y_density = [" + octave_y_density + "];\n"
        outfile_octave.write "x_density_linear = [" + octave_x_density_linear + "];\n"
        outfile_octave.write "y_density_linear = [" + octave_y_density_linear + "];\n"
        outfile_octave.write "x_vel = [" + octave_x_vel + "];\n"
        outfile_octave.write "y_vel = [" + octave_y_vel + "];\n"
        outfile_octave.close
        outfile_density.close
        outfile_vel.close
        
        stdin, stdout, stderr = Open3.popen3('/usr/bin/gnuplot generate_histograms.plot')
       
        stdin, stdout, stderr = Open3.popen3('/usr/bin/octave --silent leastsquares_radialprofiles.m')
        s = stdout.readlines
        puts s
    end
    
    #### TASK 8
    def calculate_luminosity
        ## Calculate aparent magnitude in V
        @linies.each { |l|
            x = l[f("X")]
            y = l[f("Y")]
            
            ####
            ## Source: http://en.wikipedia.org/wiki/Galaxy_morphological_classification#Numerical_Hubble_stage
            ## Spiral arms:
            # Sd (SBd) - diffuse, broken arms made up of individual stellar clusters and nebulae; very faint central bulge
            # Sm (SBm) - irregular in appearance; no bulge component
            # Im - highly irregular galaxy
            ####
            ## Source: http://cosmos.phy.tufts.edu/~zirbel/ast21/pdf/Lecture15-sm.pdf
            ## Type             B-V color
            #   Elliptical      0.95
            #   Sa              0.65
            #   Sb              0.55
            #   Sc              0.4
            #   Irregular       0.3

            if (["E","dE"].include?(l[f("Type")])) then
                color_BV = 0.95
            elsif (["B","dS","S","S0","dS0","S0a","Sa","Sab"].include?(l[f("Type")])) then
                # Considering "B" = "SB"
                color_BV = 0.65
            elsif (["Sb","Sbc"].include?(l[f("Type")])) then
                color_BV = 0.55
            elsif (["Sc","Scd","Sd"].include?(l[f("Type")])) then
                color_BV = 0.4
            elsif (["Sm","Sdm","I","Im"].include?(l[f("Type")])) then
                color_BV = 0.3
            else
                # Unknown, keep same magnitude
                color_BV = 0
            end
            
            mag_V = l[f("Apparent_magnitude_B")].to_f - color_BV
            mag_abs_B = l[f("Apparent_magnitude_B")].to_f - 5 * log10(@distancia* 100000) # Distance in 10 pc
            mag_abs_B_sun = 5.48
            luminosity_B = 10**((mag_abs_B_sun - mag_abs_B) / 2.5)
            
            l.push mag_abs_B
            @f["Absolute_magnitude_B"] = l.size - 1
            l.push mag_V
            @f["Apparent_magnitude_V"] = l.size - 1
            l.push luminosity_B
            @f["Luminosity_B"] = l.size - 1
        }
        
        
        ### Sort by magnitude V and find break magnitude
        linies_ord = @linies.sort {|a,b| a[f("Apparent_magnitude_V")]<=>b[f("Apparent_magnitude_V")]}
        mag_break = (-21.2 + 5 * log10(71/50)) + 5 * log10(@distancia * 100000) # Distance in 10 pc
        puts "Magnitude break:\t\t" + mag_break.to_s
        puts "Magnitude max:\t\t" + linies_ord[linies_ord.size-1][f("Apparent_magnitude_V")].to_s
        puts "Magnitude min:\t\t" + linies_ord[0][f("Apparent_magnitude_V")].to_s
        pos_break = 0
        linies_ord.each { |l|
            if (l[f("Apparent_magnitude_V")] >= mag_break) then
                break
            end 
            pos_break += 1       
        }
        
        if (pos_break == linies_ord.size) then
            throw Exception.new("Break magnitude not found")
        end
        
        outfile_octave = File.new("output/galaxy_magnitudes.m", "w")
        octave_y_magnitudes_left = ""
        octave_x_magnitudes_left = ""
        octave_y_magnitudes_right = ""
        octave_x_magnitudes_right = ""
        octave_y_magnitudes_rest = ""
        octave_x_magnitudes_rest = ""
        octave_y_luminosity = ""
        octave_x_luminosity = ""

        
        ### Left side (magnitude < break magnitude)
        mag_min = linies_ord[0][f("Apparent_magnitude_V")]
        mag_max = linies_ord[pos_break - 1][f("Apparent_magnitude_V")]
        width = (mag_max - mag_min) / 10
        iter = mag_min
        loop do
            linies_incloses = linies_ord.find_all { |l| l[f("Apparent_magnitude_V")] < iter}
            mag_ave = (iter + (iter + width)) / 2
            if linies_incloses == 0 then
                octave_x_magnitudes_left += mag_ave.to_s + "; "
                octave_y_magnitudes_left += "0; "
            else
                octave_x_magnitudes_left += mag_ave.to_s + "; "
                y_value = 0
                if linies_incloses.size != 0 then
                    y_value = log10(linies_incloses.size)
                end
                octave_y_magnitudes_left += y_value.to_s + "; "
            end
        
            iter += width
            break if iter > mag_max        
        end
        
        outfile_octave.write "x_magnitudes_left = [" + octave_x_magnitudes_left + "];\n"
        outfile_octave.write "y_magnitudes_left = [" + octave_y_magnitudes_left + "];\n"

        ### Right side (magnitude >= break magnitude)
        mag_min = linies_ord[pos_break][f("Apparent_magnitude_V")]
        mag_max = linies_ord[@num_galaxies - 1][f("Apparent_magnitude_V")]
        width = (mag_max - mag_min) / 10
        iter = mag_min
        last_y_value = 0
        loop do
            linies_incloses = linies_ord.find_all { |l| l[f("Apparent_magnitude_V")] < iter}
            mag_ave = (iter + (iter + width)) / 2
            y_value = 0
            if linies_incloses == 0 then
                octave_x_magnitudes_right += mag_ave.to_s + "; "
                octave_y_magnitudes_right += "0; "
            else
                if linies_incloses.size != 0 then
                    y_value = log10(linies_incloses.size)
                end
                
                # Exclude final galaxies where the number decrease because of instrument limitation (faint magnitudes)
                break if  (last_y_value != 0 and ((y_value - last_y_value) / last_y_value) < 0.06) # Increases less than 6%         
                octave_x_magnitudes_right += mag_ave.to_s + "; "
                octave_y_magnitudes_right += y_value.to_s + "; "
            end
        
            iter += width
            break if iter >= mag_max
            last_y_value = y_value
        end

        outfile_octave.write "x_magnitudes_right = [" + octave_x_magnitudes_right + "];\n"
        outfile_octave.write "y_magnitudes_right = [" + octave_y_magnitudes_right + "];\n"
        
                
        ## Excluded galaxies
        loop do
            linies_incloses = linies_ord.find_all { |l| l[f("Apparent_magnitude_V")] < iter}
            mag_ave = (iter + (iter + width)) / 2
            y_value = 0
            if linies_incloses == 0 then
                octave_x_magnitudes_rest += mag_ave.to_s + "; "
                octave_y_magnitudes_rest += "0; "
            else
                if linies_incloses.size != 0 then
                    y_value = log10(linies_incloses.size)
                end
                octave_x_magnitudes_rest += mag_ave.to_s + "; "
                octave_y_magnitudes_rest += y_value.to_s + "; "
            end
        
            iter += width
            break if iter >= mag_max
        end
        
        
        outfile_octave.write "x_magnitudes_rest = [" + octave_x_magnitudes_rest + "];\n"
        outfile_octave.write "y_magnitudes_rest = [" + octave_y_magnitudes_rest + "];\n"
        outfile_octave.write "mag_break = [" + mag_break.to_s + "];\n"
        
        
        ## Luminosity
        linies_ord = @linies.sort {|a,b| a[f("Luminosity_B")]<=>b[f("Luminosity_B")]}
        puts "Luminosity max:\t\t" + linies_ord[linies_ord.size-1][f("Luminosity_B")].to_s + " Lb(sun)"
        puts "Luminosity min:\t\t" + linies_ord[0][f("Luminosity_B")].to_s + " Lb(sun)"
        puts "Luminosity sun:\t\t3.846 x 10^26 W"
        
        lum_min = linies_ord[0][f("Luminosity_B")]
        lum_max = linies_ord[@num_galaxies - 1][f("Luminosity_B")]
        width = (lum_max - lum_min) / 100
        iter = lum_min
        loop do
            
            linies_incloses = linies_ord.find_all { |l| l[f("Luminosity_B")] < iter}
            lum_ave = (iter + (iter + width)) / 2
            if linies_incloses == 0 then
                octave_x_luminosity += lum_ave.to_s + "; "
                octave_y_luminosity += "0; "
            else
                octave_x_luminosity += lum_ave.to_s + "; "
                octave_y_luminosity += linies_incloses.size.to_s + "; "
            end
        
            iter += width
            break if iter >= lum_max        
        end
        
        outfile_octave.write "x_luminosity = [" + octave_x_luminosity + "];\n"
        outfile_octave.write "y_luminosity = [" + octave_y_luminosity + "];\n"
        outfile_octave.close
        
        
        stdin, stdout, stderr = Open3.popen3('/usr/bin/octave --silent leastsquares_AbelLuminosityFunc.m')
        s = stdout.readlines
        puts s
    end
    
    #### TASK 9
    def calculate_mass_to_light_ratio
    
        # Velocity dispersion
        w = 0
        @linies.each { |l|
            v = l[f("Velocity")]
            w += (v - @vel_caiguda)**2
        }
        w /= @num_galaxies
        w = sqrt(w)

        l_numerator = 0        
        l_denominator = 0
        harmonic_radius = 0
        for i in 0..@num_galaxies-1
            l_numerator += @linies[i][f("Luminosity_B")]
            for j in 0..i-1
                l_denominator += @linies[i][f("Luminosity_B")] * @linies[j][f("Luminosity_B")]
                x1 = @linies[i][f("X")]
                y1 = @linies[i][f("Y")]
                x2 = @linies[j][f("X")]
                y2 = @linies[j][f("Y")]
                harmonic_radius += 1 / sqrt((x1 - x2)**2 + (y1 - y2)**2)
            end
        end
        harmonic_radius /= @num_galaxies
        m2l = ((3 * PI * w**2) / (2 * 6.67428e-11 * harmonic_radius)) * (l_numerator / l_denominator)
        puts "Mass to light:\t\t" + m2l.to_s + " M(sun) / L(sun)"
    end
    
    
    ######################## PRIVATE
    private
    
    def f(field)
        return @f[field]
    end
    
    def reset_attributes
        @f = Hash.new                           # Fields
        @f["VCC"] = 0                           # Id
        @f["RA_h"] = 1                            # (1950.0)
        @f["RA_m"] = 2                            # (1950.0)
        @f["RA_s"] = 3                            # (1950.0)
        @f["DEC_g"] = 4                           # (1950.0)
        @f["DEC_m"] = 5                           # (1950.0)
        @f["DEC_s"] = 6                           # (1950.0)
        @f["Member"] = 7                          # 1:member; 2:background; 3:possible memb.  
        @f["Type"] = 8                            # U: unknown; ?: doubtful.     
        @f["Apparent_magnitude_B"] = 9
        @f["Diameter"] = 10                      # (0.1 arcmin)
        @f["Heliocentric_velocity"] = 11
        @f["Heliocentric_velocity_error"] = 12
        @vel_min = 0.0
        @vel_max = 0.0
        @vel_mitja = 0.0
        @vel2_mitja = 0.0
        @sigma = 0.0
        @num_galaxies = 0
        @mag_max = 0.0
        @mag_min = 20.0
        @mag_mitja = 0.0
    end
    
    def print_vel_attributes
        puts "Velocitat mitja:\t" + @vel_mitja.to_s + " km/s"
        puts "Desviació std (sigma):\t" + @sigma.to_s + " km/s"
        puts "Num galaxies:\t\t" + @num_galaxies.to_s
    end
    
    ## Cluster attributes
    def calculate_attributes
        reset_attributes
        @linies.each { |l|
            if (l[f("Heliocentric_velocity")].to_f > @vel_max) then
                @vel_max = l[f("Heliocentric_velocity")].to_f
            end
            if (l[f("Heliocentric_velocity")].to_f < @vel_min) then
                @vel_min = l[f("Heliocentric_velocity")].to_f
            end
            @vel_mitja += l[f("Heliocentric_velocity")].to_f
            @vel2_mitja += l[f("Heliocentric_velocity")].to_f ** 2
            

            # Magnitude
            if (l[f("Apparent_magnitude_B")].to_f > @mag_max) then
                @mag_max = l[f("Apparent_magnitude_B")].to_f
            end
            if (l[f("Apparent_magnitude_B")].to_f < @mag_min) then
                @mag_min = l[f("Apparent_magnitude_B")].to_f
            end
            @mag_mitja += l[f("Apparent_magnitude_B")].to_f
            @num_galaxies += 1
        }

        @vel_mitja = @vel_mitja.to_f / @num_galaxies
        @vel2_mitja = @vel2_mitja.to_f / @num_galaxies        
        @mag_mitja /= @num_galaxies
        
        @sigma = 0
        @linies.each { |l|
            @sigma += (l[f("Heliocentric_velocity")].to_f  - @vel_mitja)**2
            
        }
        @sigma /= (@num_galaxies)
        @sigma = Math.sqrt(@sigma)        
        #@sigma = Math.sqrt(@vel2_mitja - @vel_mitja**2)
    end
    

    ############# [start] Filter methods
    def filter_zero_or_no_member
        noves_linies = []
        @linies.each { |l|
            if ((l[f("Member")] != "2") and (l[f("Heliocentric_velocity")] != "0") and (l[f("Apparent_magnitude_B")] != "0") and (l[f("Type")] != "U") and (l[f("Type")] != "?")) then
                noves_linies.push l
            end
        }
        @linies = noves_linies
        calculate_attributes
    end
    
    def filter_3sigma
        noves_linies = []
        @linies.each { |l|
            if (l[f("Heliocentric_velocity")].to_f < 3 * @sigma + @vel_mitja and l[f("Heliocentric_velocity")].to_f > @vel_mitja -3 * @sigma) then
                noves_linies.push l
            end
        }
        @linies = noves_linies
        calculate_attributes
    end
    ############# [end] Filter methods
end



c = Cumul.new("input/VIRGODOS.txt")
puts ":: Neteja de dades inicials"
c.clean
puts ":: Càlculs"
c.calculate_vel_caiguda
c.calculate_xy
c.calculate_ellipticity_and_orientation
c.calculate_lambda
c.calculate_rings
c.calculate_luminosity
c.calculate_mass_to_light_ratio
c.save("output/VIRGODOS_final_data.txt")
stdin, stdout, stderr = Open3.popen3('/usr/bin/gnuplot generate_final_galaxies_and_ellipsoid.plot')



